Attribute VB_Name = "StruttureBMP"
'================================================================
' Descrizione.....: Strutture, tipi e costanti, piu' comunemente
'                   usati, per trafficare con BMP (Ref: Bmp.txt).
'                   Routines e API per la creazione e la gestione
'                   dei DIB e dei PictureBoxes.
' Nome dei Files..: StruttureBMP.bas
' Data............: 5/11/2000
' Aggiornamento...: 18/2/2001
' Versione........: 1.0 a 32 bits
' Sistema.........: Visual Basic 6.0
'                   sotto Windows NT 4.0 (SP 3).
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
'================================================================
'
'   Le routines comprese in questo modulo e
'   di uso piu' frequente sono:
'    LeggeAreaPB        ' Vedere la descrizione
'    LeggeBitMap3PB     ' all' interno delle
'    ScriveBitMap3PB    ' stesse routines.
'    Pix3ToRGB          '
'    RGBToPix3          '
'    LeggeBitMapPBInfo  '
'
'   Nota1:  nelle routines di questo modulo le matrici sono definite come
'            MPixRGB():  matrici [Long] di pixels in formato RGB.
'            MPix3():    matrici [Bytes] contenenti, a 3 a 3, i
'                        colori dei pixels nei bytes successivi.
'            MPix4():    matrici [Long] contenenti i colori RGB
'                        dei pixels con il byte del rosso scambiato
'                        con quello del blu.
'            MPix():     matrici [Bytes] di pixels restituite dalle
'                        API GetBitmapBits e SetBitmapBits. Il formato
'                        di queste matrici dipende dalla profondita'
'                        di colore e dalle impostazioni della scheda
'                        grafica.
'            Tutte le matrici iniziano dagli indici M(0, 0).
'
'   Nota2:  l' orientamento dell' asse Y, nelle routines Legge_PictureBox
'           LeggeBitMap3PB, ScriveBitMap3PB, LeggeBitMap4PB e ScriveBitMap4PB,
'           puo' essere invertito con: bi.bmiHeader.biHeight = -I_Height
'
Option Explicit
'
' Strutture e tipi BMP: -------------------------------------------------------------
'
'   For Windows 3.0:
'    BITMAPFILE{
'       BITMAPFILEHEADER        bmfHeader
'       BITMAPINFO{             bi
'           BITMAPINFOHEADER    bmiHeader
'           RGBQUAD()           bmiColors
'       }
'       BYTE()                  Pixels
'    }
'
Private Type BITMAPFILEHEADER_Type
    bfType          As String * 2   ' The string BM (hex value &H424D).
    bfSize          As Long         ' The size of the file, measured in [Bytes].
    bfReserved1     As Integer      ' Not used, set to zero.
    bfReserved2     As Integer      ' Not used, set to zero.
    bfOffBits       As Long         ' The start offset of the bitmap data in the file.
End Type
'
Private Type BITMAPINFOHEADER_Type
    biSize          As Long         ' 40 (the size of this structure).
    biWidth         As Long         ' The width of the bitmap in pixels.
    biHeight        As Long         ' The height of the bitmap in pixels.
    biPlanes        As Integer      ' 1 (DIBs always have one plane).
    biBitCount      As Integer      ' 1 for monochrome, 4 for 16 colors,
                                    ' 8 for 256 color, 24 for 24-bit RGB color.
    biCompression   As Long         ' Specifies the type of compression for compressed
                                    ' bitmap. It can be one of the following values:
                                    ' BI_RGB:   BitMap is not compressed.
                                    ' BI_RLE8:  encoded format with 8 [BitsPerPixel].
                                    ' BI_RLE4:  encoded format with 4 [BitsPerPixel].
    biSizeImage     As Long         ' The size of the image in bytes.
                                    ' May be zero if biCompression is BI_RGB.
    biXPelsPerMeter As Long         ' Number of horizontal pixels per meter for
                                    ' which this DIB was designed.
    biYPelsPerMeter As Long         ' Number of vertical pixels per meter for
                                    ' which this DIB was designed.
    biClrUsed       As Long         ' Number of entries in the DIB color table
                                    ' that are actually used. Zero if all are used.
                                    ' Must be zero or the actual color table size
                                    ' for packed DIBs
    biClrImportant  As Long         ' Number of entries in the DIB color table that
                                    ' are important. Zero to specify all colors.
End Type
'
Private Type RGBQUAD_Type
    rgbBlue As Byte
    rgbGreen As Byte
    rgbRed As Byte
    rgbReserved As Byte
End Type
'
Private Type BITMAPINFO_Type
    bmiHeader As BITMAPINFOHEADER_Type
    bmiColors(0 To 255) As RGBQUAD_Type ' E' dimensionato al massimo numero
                                        ' possibile di colori perche' l' uso
                                        ' di array dinamici non e' consentito
                                        ' nell' API GetDIBits.
End Type
'
Private Type BITMAP_Type
    bmType As Long
    bmWidth As Long
    bmHeight As Long
    bmWidthBytes As Long
    bmPlanes As Integer
    bmBitsPixel As Integer
    bmBits As Long
End Type
'
'------------------------------------------------------------------------------------
'
'   For OS/2 Presentation Manager:
'    BITMAPFILE{
'       BITMAPFILEHEADER        bmfHeader
'       BITMAPCOREINFO{         bci
'           BITMAPCOREHEADER    bmciHeader
'           RGBTRIPLE()         bmciColors
'       }
'       BYTE()                  Pixels
'    }
'
Private Type BITMAPCOREHEADER_Type
    bcSize          As Long         ' 12 (the size of this structure).
    bcWidth         As Integer      ' The width of the bitmap in pixels.
    bcHeight        As Integer      ' The height of the bitmap in pixels.
    bcPlanes        As Integer      ' 1 (DIBs always have one plane).
    bcBitCount      As Integer      ' 1 for monochrome, 4 for 16 colors,
                                    ' 8 for 256 color, 24 for 24-bit RGB color.
End Type
'
Private Type RGBTRIPLE_Type
    rgbtBlue As Byte
    rgbtGreen As Byte
    rgbtRed As Byte
End Type
'
Private Type BITMAPCOREINFO_Type
    bmciHeader As BITMAPINFOHEADER_Type
    bmciColors() As RGBTRIPLE_Type
End Type
'
' Costanti e API: -------------------------------------------------------------------
'
Private Const BI_RGB& = 0&
Private Const BI_RLE8& = 1&
Private Const BI_RLE4& = 2&
'
'------------------------------------------------------------------------------------
Private Type RECT_Type
    Left  As Long
    Top  As Long
    Right  As Long
    Bottom  As Long
End Type
'
' DC related API
Private Declare Function CreateCompatibleDC Lib "gdi32" _
(ByVal hDC As Long) As Long
Private Declare Function GetWindowDC Lib "user32" _
(ByVal hWnd As Long) As Long
Private Declare Function DeleteDC Lib "gdi32" _
(ByVal hDC As Long) As Long
'
' Graphics related API
Private Const DIB_RGB_COLORS& = 0
Private Const DIB_PAL_COLORS& = 1
Private Const SRCCOPY& = &HCC0020
'
Private Declare Function GetDIBits Lib "gdi32" (ByVal hDC As Long _
, ByVal hBitmap As Long, ByVal nStartScan As Long, ByVal nNumScans As Long _
, lpBits As Any, BitsInfo As BITMAPINFO_Type, ByVal wUsage As Long) As Long
'
Private Declare Function SetDIBits Lib "gdi32" (ByVal hDC As Long _
, ByVal hBitmap As Long, ByVal nStartScan As Long, ByVal nNumScans As Long _
, lpBits As Any, BitsInfo As BITMAPINFO_Type, ByVal wUsage As Long) As Long
'
Private Declare Function SetDIBitsToDevice Lib "gdi32" (ByVal hDC As Long _
, ByVal X As Long, ByVal Y As Long, ByVal DX As Long, ByVal DY As Long _
, ByVal SrcX As Long, ByVal SrcY As Long, ByVal Scan As Long, ByVal NumScans As Long _
, lpBits As Any, lpBI As BITMAPINFO_Type, ByVal wUsage As Long) As Long
'
Private Declare Function StretchDIBits Lib "gdi32" (ByVal hDC As Long _
, ByVal DestX As Long, ByVal DestY As Long, ByVal wDestWidth As Long _
, ByVal wDestHeight As Long, ByVal SrcX As Long, ByVal SrcY As Long _
, ByVal wSrcWidth As Long, ByVal wSrcHeight As Long, ByVal lpBits& _
, BitsInfo As BITMAPINFO_Type, ByVal wUsage As Long, ByVal dwRop&) As Long
'
Private Declare Function GetBitmapBits Lib "gdi32" (ByVal hBitmap As Long _
, ByVal dwCount As Long, lpBits As Any) As Long
'
Private Declare Function SetBitmapBits Lib "gdi32" (ByVal hBitmap As Long _
, ByVal dwCount As Long, lpBits As Any) As Long
'
Private Declare Function GetDIBColorTable Lib "gdi32" (ByVal hDC As Long _
, ByVal un1 As Long, ByVal un2 As Long, pRGBQuad As RGBQUAD_Type) As Long
'
Private Declare Function SetDIBColorTable Lib "gdi32" (ByVal hDC As Long _
, ByVal un1 As Long, ByVal un2 As Long, pcRGBQuad As RGBQUAD_Type) As Long
'
' General attribute related API
Private Declare Function GetClientRect Lib "user32" _
(ByVal hWnd As Long, lpRect As RECT_Type) As Long
'
Private Declare Function GetObjectAPI& Lib "gdi32" Alias "GetObjectA" _
(ByVal hObject As Long, ByVal nCount As Long, lpObject As Any)
'
' Memory allocation related API
Private Const GMEM_MOVEABLE& = 2
'
Private Declare Function GlobalAlloc Lib "kernel32" _
(ByVal wFlags As Long, ByVal lMem&) As Long
Private Declare Function GlobalLock Lib "kernel32" _
(ByVal hMem As Long) As Long
Private Declare Function GlobalUnlock Lib "kernel32" _
(ByVal hMem As Long) As Long
Private Declare Function GlobalFree Lib "kernel32" _
(ByVal hMem As Long) As Long
'
Private Declare Sub MoveMemory Lib "kernel32" Alias "RtlMoveMemory" _
(pDest As Any, pSource As Any, ByVal dwLength As Long)
Public Function Legge_PictureBox(ByVal PBSrc As PictureBox, ByVal BxP& _
    , bi As BITMAPINFO_Type, ByRef lpBits&, ByRef hMem& _
    , Optional ByVal XPPM& = 0 _
    , Optional ByVal YPPM& = 0 _
    , Optional ByRef BytesNeeded& _
    , Optional ByRef BytesPerScanLine& _
    , Optional ByRef I_Width&, Optional ByRef I_Height&) As Boolean
'
'   Legge il PictureBox PBSrc e crea un DIB a BxP [BitsPerPixel]
'   definito nelle variabili bi, lpBits e hMem.
'   Il blocco di memoria hMem va' liberato dopo l' uso.
'   XPPM e YPPM sono il numero di pixels per metro orizzontale e
'   verticale su cui impostare il DIB: se non conosciuti possono
'   essere lasciati a 0.
'   lpBits e' l' indirizzo del primo byte di hMem.
'   BytesNeeded e' la dimensione, in [Bytes], del blocco di memoria hMem.
'   BytesPerScanLine e' la larghezza vera, in [Bytes], della riga.
'   I_Width e I_Height sono, rispettivamente, la larghezza e l' altezza
'   dell' immagine [Pixels].
'
    Dim R1&, M$
    Dim hDC_PB&, hDC_Mem&
    Dim Rect As RECT_Type
'
    On Error GoTo Legge_PictureBox_ERR
'
    ' Se gia' occupato, libera il blocco
    ' di memoria contenente i dati correnti
    ' del DIB:
    If hMem <> 0 Then
        R1 = GlobalUnlock(hMem)
        hMem = GlobalFree(hMem)
    End If
'
    ' Create a temporary memory DC and select into it
    ' the background picture of the PBSrc Picture Box:
    hDC_PB = GetWindowDC(PBSrc.hWnd)
    hDC_Mem = CreateCompatibleDC(hDC_PB)
'
    ' Get the size of the picture bitmap:
    R1 = GetClientRect(PBSrc.hWnd, Rect)
    I_Width = Abs(Rect.Right - Rect.Left)
    I_Height = Abs(Rect.Bottom - Rect.Top)
'
    ' Fill the BITMAPINFOHEADER for the desired DIB:
    bi.bmiHeader.biSize = Len(bi.bmiHeader)
    bi.bmiHeader.biWidth = I_Width
    bi.bmiHeader.biHeight = I_Height
    bi.bmiHeader.biPlanes = 1           ' ColorPlanes
    bi.bmiHeader.biBitCount = CInt(BxP) ' BitsPerPixel * ColorPlanes
    bi.bmiHeader.biCompression = BI_RGB
    bi.bmiHeader.biSizeImage = 0
    bi.bmiHeader.biXPelsPerMeter = XPPM
    bi.bmiHeader.biYPelsPerMeter = YPPM
    bi.bmiHeader.biClrUsed = 0
    bi.bmiHeader.biClrImportant = 0
'
    ' Calculate the number of bytes needed to store the image:
    BytesNeeded = MemBuffer(bi.bmiHeader.biBitCount _
                          , I_Width, I_Height _
                          , BytesPerScanLine)
    If BytesNeeded = 0 Then
        M$ = "Formato grafico sconosciuto." & vbNewLine
        Err.Raise 1001
    End If
'
    ' Allocate a buffer to hold the bitmap bits:
    hMem = GlobalAlloc(GMEM_MOVEABLE, BytesNeeded)
'
    If hMem <> 0 Then
        lpBits = GlobalLock(hMem)
'
        ' L' immagine deve essere in background
        ' Non necessario se PBSrc.Autoredraw = True:
        If Not PBSrc.AutoRedraw Then
            If Not BackTrasPB(PBSrc) Then Err.Raise 1001, , "BackTrasPB"
        End If
'
        ' Get the bits that make up the image:
        R1 = GetDIBits(hDC_Mem, PBSrc.Picture, 0, I_Height _
        , ByVal lpBits, bi, DIB_RGB_COLORS)
        If R1 = 0 Then
            M$ = "Errore in GetDIBits." & vbNewLine
            Err.Raise 1001
        End If
    Else
        Err.Raise 1001
    End If
'
'
Legge_PictureBox_ERR:
    Legge_PictureBox = (Err = 0)
    If Err <> 0 Then
        M$ = M$ & "Errore N. " & Err.Number & vbNewLine & Err.Description
        MsgBox M$, vbCritical, " Errore in Legge_PictureBox"
    End If
'
    R1 = DeleteDC(hDC_Mem)
'
'
'
End Function
Public Function LeggeAreaPB(ByVal PBSrc As PictureBox, ByVal BxP&, MPixRGB&() _
    , Optional ByVal x1& = 0, Optional ByVal y1& = 0 _
    , Optional ByVal x2& = 0, Optional ByVal y2& = 0 _
    , Optional ByRef PBWidth&, Optional ByRef PBHeight&) As Boolean
'
'   Ritorna, nella matrice MPixRGB(Xn, Ym), i valori RGB dei pixels
'   compresi nell' area X1 -> X2 e Y1 -> Y2 (estremi inclusi)
'   del PictureBox PBSrc.  L' immagine viene prima convertita
'   alla profondita' di colore BxP [BitsPerPixel].
'   Il punto (X1, Y1) e' l' angolo inferiore sinistro dell' area
'   che si vuole leggere e (X2, Y2) l' angolo superiore destro.
'   Il punto MPixRGB(0, 0) corrisponde all' angolo inferiore sinistro.
'   Deve essere:
'   0 <= X1 <= X2 <= PBWidth - 1  e  0 <= Y1 <= Y2 <= PBHeight - 1.
'   Nessun controllo viene effettuato.  I valori PBWidth e PBHeight
'   possono essere, eventualmente, letti con questa routine effettuando
'   una prima chiamata con lettura di un solo Pixel ( X1 = Y1 = X2 = Y2 = 0).
'
'   Nota:   questa versione di LeggeAreaPB lavora con un orientamento
'           dell' asse Y crescente dal basso verso l' alto, compatibile
'           con le routines LeggeBitMap3PB e ScriveBitMap3PB (e con la
'           disposizione dei bytes all' indirizzo lpBits).
'
    Dim C&, C1&, C2&, Cl&, Cl1&, Col&(), TCol&()
    Dim XIP&, InX0&, InX&, BnX&, XBMP&
    Dim YIP&, InY&, BnY&
    Dim Pi3B&, InB0&, InB&
    Dim bi As BITMAPINFO_Type, lpBits&, hMem&, R1&
'
    On Error GoTo LeggeAreaPB_ERR
'
    ' Legge il DIB del PictureBox:
    If Not Legge_PictureBox(PBSrc, BxP, bi, lpBits, hMem _
                    , , , , XBMP, PBWidth, PBHeight) Then
        Err.Raise 1001, , "LeggeAreaPB/Legge_PictureBox"
    End If
'
    ReDim MPixRGB(0 To x2 - x1, 0 To y2 - y1)
'
    If BxP <= 8 Then
        ' Prepara la tavola dei colori:
        TCol() = biToColTab(bi)
    End If
'
    ' N. di pixels che stanno in 3 bytes:
    Pi3B = Switch(BxP = 1, 24, BxP = 4, 6 _
                , BxP = 8, 3, BxP = 24, 1)
    ReDim Col(0 To Pi3B - 1) ' Vettore dei colori dei pixels che stanno in 3 bytes.
'
    ' Calcolo il byte di inizio lettura
    ' all' interno della riga:
    InX0 = CLng(3! * CSng(x1) / CSng(Pi3B))
    ' Calcolo il bit di inizio
    ' all' interno del primo byte:
    If BxP = 1 Then
        InB0 = x1 Mod 8
    ElseIf BxP = 4 Then
        InB0 = x1 Mod 2
    Else
        InB0 = 0
    End If
'
    YIP = y1
    Do
        InY = YIP
        BnY = InY * XBMP + lpBits   ' Posizione, nel DIB, del byte corrispondente
                                    ' all' inizio della InYma riga.
        InB = InB0
        InX = InX0
        XIP = x1
        Do
            BnX = BnY + InX     ' Posizione, nel DIB, del byte corrispondente a InX
                                ' all' interno della InYma riga.
'
            ' Leggo dal DIB 3 bytes per volta:
            MoveMemory Cl, ByVal BnX, 3
            Cl = Cl And &HFFFFFF
            InX = InX + 3
'
            ' Leggo il colore dei pixels contenuti nei 3 bytes.
            ' Nel caso di 1, 4 e 8 BitsPerPixel il valore letto
            ' dal file e' un indice nella tabella dei colori TCol():
            If BxP = 1 Then
                ' 24 pixels da leggere:
                C = 23
                For C1 = 2 To 0 Step -1
                    Cl1 = (Cl \ (&H100 ^ C1)) And &HFF
                    For C2 = 0 To 7
                        If (Cl1 And (2 ^ C2)) = 0 Then
                            Col(C) = TCol(0)
                        Else
                            Col(C) = TCol(1)
                        End If
                        C = C - 1
                    Next C2
                Next C1
'
            ElseIf BxP = 4 Then
                ' 6 pixels da leggere:
                Col(4) = TCol(Cl \ &H100000)
                Col(5) = TCol(Cl \ &H10000 And &HF)
                Col(2) = TCol(Cl \ &H1000 And &HF)
                Col(3) = TCol(Cl \ &H100 And &HF)
                Col(0) = TCol(Cl \ &H10 And &HF)
                Col(1) = TCol(Cl And &HF)
'
            ElseIf BxP = 8 Then
                ' 3 pixels da leggere:
                Col(2) = TCol(Cl \ &H10000)
                Col(1) = TCol(Cl \ &H100 And &HFF)
                Col(0) = TCol(Cl And &HFF)
'
            ElseIf BxP = 24 Then
                ' 1 solo pixel da leggere direttamente dal DIB:
                Col(0) = RGB(Cl \ &H10000, (Cl \ &H100) And &HFF, Cl And &HFF)
            End If
'
            For C = InB To Pi3B - 1
                MPixRGB(XIP - x1, YIP - y1) = Col(C)
                XIP = XIP + 1
                If XIP > x2 Then Exit Do
            Next C
            InB = 0
        Loop While XIP <= x2
'
        DoEvents
'
        YIP = YIP + 1
    Loop While YIP <= y2
'
'
LeggeAreaPB_ERR:
    LeggeAreaPB = (Err = 0)
'
    ' Libera il blocco di memoria contenente
    ' i dati correnti del DIB:
    If hMem <> 0 Then
        R1 = GlobalUnlock(hMem)
        hMem = GlobalFree(hMem)
    End If
'
'
'
End Function

Public Sub ImmGrigia(bi As BITMAPINFO_Type)
'
'   Converte la tavola dei colori, definita
'   in bmiColors(), in toni di grigio:
'
    Dim BxP&, M$
    Dim C&, R&, G&, B&, Gr&
'
    BxP = bi.bmiHeader.biBitCount
    If BxP > 8 Then
        M$ = "il Dib non ha Color Table" & vbNewLine
        M$ = M$ & "e non puo' essere convertito" & vbNewLine
        M$ = M$ & "in toni di grigio." & vbNewLine
        MsgBox M$, vbInformation, " ImmGrigia"
        Exit Sub
    End If
'
    For C = 0 To (2 ^ BxP) - 1
        R = bi.bmiColors(C).rgbRed
        G = bi.bmiColors(C).rgbGreen
        B = bi.bmiColors(C).rgbBlue
'
        Gr = (R + G + B) / 3
        bi.bmiColors(C).rgbRed = Gr
        bi.bmiColors(C).rgbGreen = Gr
        bi.bmiColors(C).rgbBlue = Gr
    Next C
'
'
'
End Sub
Public Function biToColTab(bi As BITMAPINFO_Type _
    , Optional ByRef NCol&) As Long()
'
'   Prepara e ritorna la tavola dei colori
'   RGB corrispondente a bi.bmiColors():
'
    Dim C&
'
    NCol = 2 ^ bi.bmiHeader.biBitCount
    ReDim biCT(0 To NCol - 1) As Long
'
    For C = 0 To NCol - 1
        biCT(C) = RGB(bi.bmiColors(C).rgbRed _
                    , bi.bmiColors(C).rgbGreen _
                    , bi.bmiColors(C).rgbBlue)
    Next C
'
    biToColTab = biCT()
'
'
'
End Function

Public Function LunghezzaFileBMP(ByVal BxP& _
    , ByVal F_Width&, ByVal F_Height&) As Long
'
'   Ritorna la lunghezza, in [Bytes], di un file .BMP atto
'   ad ospitare un' immagine di F_Width x F_Height [Pixels]
'   tenendo conto del numero di colori BxP [BitsPerPixel]:
'
    Dim LFB&
'
    ' BITMAPFILEHEADER:
    LFB = 14
'
    ' + BITMAPINFOHEADER:
    LFB = LFB + 40
'
    ' + Tabella dei colori:
    Select Case BxP
        Case 1
        ' Immagine Bianco/Nero:
        LFB = LFB + 8
        Case 4
        ' Immagine 16 colori:
        LFB = LFB + 64
        Case 8
        ' Immagine 256 colori:
        LFB = LFB + 1024
        Case 24
        ' Immagine TrueColor:
        LFB = LFB + 0
    End Select
'
    ' + Tabella dei dati:
    LFB = LFB + MemBuffer(BxP, F_Width, F_Height)
'
'
    LunghezzaFileBMP = LFB  ' [Bytes].
'
'
'
End Function
Public Sub PB_BxP(ByVal picPB As PictureBox)
'
'   Mostra la profondita' di colore della Picture e
'   dell' Image del PictureBox picPB:
'
    Dim R1&, bm As BITMAP_Type, M$
'
    R1 = GetObjectAPI(picPB.Picture, Len(bm), bm)
    M$ = "Picture: " & bm.bmBitsPixel & " [BitsPerPixel]" & vbNewLine
    R1 = GetObjectAPI(picPB.Image, Len(bm), bm)
    M$ = M$ & "Image:   " & bm.bmBitsPixel & " [BitsPerPixel]"
'
    MsgBox M$, vbInformation, "BxP di: " & picPB.Name
'
'
'
End Sub

Private Function ScanAlign(ByVal Valore&) As Long
'
'   Ritorna un multiplo di 4 uguale o immediatamente
'   superiore a Valore&.
'
    ScanAlign = (Valore + 3&) And &HFFFFFFFC
'
'
'
End Function
Private Function MemBuffer(ByVal BxP& _
    , ByVal I_Width&, ByVal I_Height& _
    , Optional ByRef BytesPerScanLine&) As Long
'
'   Ritorna il numero di Bytes necessario a memorizzare una
'   immagine di I_Width x I_Height [Pixels] tenendo conto del
'   numero di colori BxP [BitsPerPixel]. Imposta anche la variabile
'   BytesPerScanLine sulla larghezza vera, in [Bytes], della riga.
'   Ritorna 0 se il numero di colori e' sconosciuto.
'
    Select Case BxP
        Case 1
        BytesPerScanLine = (I_Width + 7) \ 8
        Case 4
        BytesPerScanLine = (I_Width + 1) \ 2
        Case 8
        BytesPerScanLine = I_Width
        Case 16
        BytesPerScanLine = I_Width * 2
        Case 24
        BytesPerScanLine = I_Width * 3
        Case 32
        BytesPerScanLine = I_Width * 4
        Case Else
        MemBuffer = 0
        Exit Function
    End Select
'
    BytesPerScanLine = ScanAlign(BytesPerScanLine)
    MemBuffer = BytesPerScanLine * I_Height
'
'
'
End Function


Public Function BackTrasPB(ByVal PBSrc As PictureBox) As Boolean
'
'   Blocca l' immagine di PBSrc come Background BitMap.
'   Funziona anche con scheda video impostata a profondita'
'   di colore = 8 [BitsPerPixel]. PBSrc.Picture = PBSrc.Image
'   in alcune condizioni non funziona.
'
    Dim FileN$
'
    On Error GoTo BackTrasPB_ERR
'
    FileN$ = TempFile$
    SavePicture PBSrc.Image, FileN$
    PBSrc.Picture = LoadPicture()
    PBSrc.Picture = LoadPicture(FileN$)
    If FileExists(FileN$) Then Kill FileN$
'
'
BackTrasPB_ERR:
    BackTrasPB = (Err = 0)
'
'
'
End Function

Public Function LeggeBitMapPBInfo(ByVal PBSrc As PictureBox _
    , Optional ByRef PB_Width&, Optional ByRef PB_Height& _
    , Optional ByRef PB_BxP&, Optional ByRef PB_ScanWidth& _
    , Optional ByRef PB_lpBits&) As Boolean
'
'   Legge e ritorna i parametri dell' immagine del PictureBox PBSrc.
'    PB_Width:      larghezza dell' immagine [Pixels].
'    PB_Height:     altezza dell' immagine [Pixels].
'    PB_BxP:        profondita' di colore della Picture [BitsPerPixel].
'                   Attenzione a distinguere fra questa profondita'
'                   e quella dell' Image.
'    PB_ScanWidth:  larghezza vera di una riga dell' immagine [Bytes].
'    PB_lpBits:     indirizzo dei dati della BitMap.
'
    Dim R1&, bm As BITMAP_Type
'
    R1 = GetObjectAPI(PBSrc.Image, Len(bm), bm)
    PB_Width = bm.bmWidth
    PB_Height = bm.bmHeight
    'PB_BxP = CLng(bm.bmBitsPixel)  ' BxP dell' Image.
    PB_ScanWidth = bm.bmWidthBytes
    PB_lpBits = bm.bmBits
'
    R1 = GetObjectAPI(PBSrc.Picture, Len(bm), bm)
    PB_BxP = CLng(bm.bmBitsPixel)   ' BxP della Picture.
'
    LeggeBitMapPBInfo = (R1 = 24)
'
'
'
End Function

Public Function LeggeBitMapPB(ByVal PBSrc As PictureBox, MP() As Byte _
    , Optional ByRef PB_Width&, Optional ByRef BxP&) As Boolean
'
'   Legge l' immagine del PictureBox PBSrc nella matrice MP().
'   Se la profondita' di colore della scheda grafica e' impostata
'   a 24 [BitsPerPixel] i bytes nella matrice sono a 3 a 3 i colori
'   dei pixels di PBSrc.
'   E.g.    MP(0, 0) = componente blu   del primo pixel della prima riga in alto.
'           MP(1, 0) = componente verde  "    "     "     "     "     "   "   "
'           MP(2, 0) = componente rossa  "    "     "     "     "     "   "   "
'   Ritorna anche:
'    PB_Width:  larghezza vera di una riga dell' immagine [Pixels].
'    BxP:       profondita' di colore dell' immagine [BitsPerPixel].
'
'   Nota:   l' uso dell' API GetBitmapBits e' sconsigliabile perche'
'           lavora con "Device-Dependent Bitmaps" che variano in funzione
'           della profondita' di colore e della scheda grafica impiegata.
'           Molto meglio e' l' uso dei DIB.  La routine LeggeBitMap3PB e'
'           da preferirsi.
'
    Dim XBMP&, PB_Height&
'
    On Error GoTo LeggeBitMapPB_ERR
'
    If Not LeggeBitMapPBInfo(PBSrc, PB_Width, PB_Height, BxP, XBMP) Then Err.Raise 1001
    ReDim MP(0 To XBMP - 1, 0 To PB_Height - 1)
'
    If GetBitmapBits(PBSrc.Image, XBMP * PB_Height, MP(0, 0)) = 0 Then Err.Raise 1001
'
'
LeggeBitMapPB_ERR:
    LeggeBitMapPB = (Err = 0)
'
'
'
End Function

Public Function ScriveBitMapPB(MP() As Byte, ByVal PBDst As PictureBox) As Boolean
'
'   Scrive la matrice MP() sull' immagine del PictureBox PBSrc.
'   Se la profondita' di colore della scheda grafica e' impostata
'   a 24 [BitsPerPixel] i bytes nella matrice sono a 3 a 3 i colori
'   dei pixels da scrivere su PBDst.
'   E.g.    MP(0, 0) = componente blu   del primo pixel della prima riga in alto.
'           MP(1, 0) = componente verde  "    "     "     "     "     "   "   "
'           MP(2, 0) = componente rossa  "    "     "     "     "     "   "   "
'
'   Nota:   l' uso dell' API SetBitmapBits e' sconsigliabile perche'
'           lavora con "Device-Dependent Bitmaps" che variano in funzione
'           della profondita' di colore e della scheda grafica impiegata.
'           Molto meglio e' l' uso dei DIB.  La routine ScriveBitMap3PB e'
'           da preferirsi.
'
    Dim dwCount&
'
    On Error GoTo ScriveBitMapPB_ERR
'
    dwCount = ((UBound(MP, 1) + 1) * (UBound(MP, 2) + 1))
    If SetBitmapBits(PBDst.Image, dwCount, MP(0, 0)) = 0 Then Err.Raise 1001
'
'
ScriveBitMapPB_ERR:
    ScriveBitMapPB = (Err = 0)
'
'
'
End Function

Public Sub Pix3ToRGB(MPix3() As Byte, MPixRGB&())
'
'   Converte i bytes della matrice MPix3(), contenente a 3 a 3 i colori dei
'   pixels, in una matrice MPixRGB() con gli stessi colori in formato RGB.
'   La matrice MPix3() ha il formato restituito da LeggeBitMap3PB.
'
    Dim X&, Y&, BnX&, I_Width_1&, I_Height_1&
'
    I_Width_1 = ((UBound(MPix3, 1) + 1) \ 3) - 1
    I_Height_1 = UBound(MPix3, 2)
    ReDim MPixRGB(0 To I_Width_1, 0 To I_Height_1)
'
    For Y = 0 To I_Height_1
        For X = 0 To I_Width_1
            BnX = 3 * X         ' Rosso            ' Verde            ' Blu
            MPixRGB(X, Y) = RGB(MPix3(BnX + 2, Y), MPix3(BnX + 1, Y), MPix3(BnX, Y))
        Next X
    Next Y
'
'
'
End Sub

Public Sub RGBToPix3(MPixRGB&(), MPix3() As Byte)
'
'   Converte una matrice MPixRGB(), contenente colori di pixels in
'   formato RGB, in una matrice di bytes MPix3() contenente a 3 a 3,
'   nei bytes successivi, i colori degli stessi pixels.
'   La matrice MPix3() risultante e' di formato adatto ad essere
'   passata a ScriveBitMap3PB.
'
    Dim X&, Y&, BnX&, I_Width_1&, I_Height_1&, XBMP&
'
    I_Width_1 = UBound(MPixRGB, 1)
    I_Height_1 = UBound(MPixRGB, 2)
    XBMP = ScanAlign(3 * (I_Width_1 + 1))
    ReDim MPix3(0 To XBMP - 1, 0 To I_Height_1)
'
    For Y = 0 To I_Height_1
        For X = 0 To I_Width_1
            BnX = 3 * X
            MPix3(BnX + 2, Y) = MPixRGB(X, Y) And &HFF            ' Rosso.
            MPix3(BnX + 1, Y) = (MPixRGB(X, Y) \ &H100) And &HFF  ' Verde.
            MPix3(BnX, Y) = (MPixRGB(X, Y) \ &H10000) And &HFF    ' Blu.
            
        Next X
    Next Y
'
'
'
End Sub

Public Function LeggeBitMap3PB(ByVal PBSrc As PictureBox, MPix3() As Byte _
    , Optional ByRef I_Width&, Optional ByRef I_Height& _
    , Optional ByRef BxP&) As Boolean
'
'   Legge l' immagine del PictureBox PBSrc nella matrice MPix3().
'   I bytes nella matrice sono a 3 a 3 i colori dei pixels di PBSrc.
'   E.g.    MPix3(0, 0) = componente blu   del primo pixel dell' ultima riga in basso.
'           MPix3(1, 0) = componente verde  "    "     "     "     "     "    "   "
'           MPix3(2, 0) = componente rossa  "    "     "     "     "     "    "   "
'   Ritorna anche:
'    I_Width:   larghezza vera di una riga dell' immagine [Pixels].
'               Puo' essere 3 * I_Width <= UBound(MPix3, 1) + 1.
'    I_Height:  altezza dell' immagine [Pixels].
'    BxP:       profondita' di colore dell' immagine [BitsPerPixel]
'               sempre = 24.
'
    Dim R1&, M$
    Dim hDC_PB&, hDC_Mem&, BytesNeeded&
    Dim Rect As RECT_Type, XBMP&
    Dim bi As BITMAPINFO_Type
'
    On Error GoTo LeggeBitMap3PB_ERR
'
    ' Create a temporary memory DC and select into it
    ' the background picture of the PBSrc Picture Box:
    hDC_PB = GetWindowDC(PBSrc.hWnd)
    hDC_Mem = CreateCompatibleDC(hDC_PB)
'
    ' Get the size of the picture bitmap:
    R1 = GetClientRect(PBSrc.hWnd, Rect)
    I_Width = Abs(Rect.Right - Rect.Left)
    I_Height = Abs(Rect.Bottom - Rect.Top)
'
    ' Fill the BITMAPINFOHEADER for the desired DIB:
    BxP = 24
    bi.bmiHeader.biSize = Len(bi.bmiHeader)
    bi.bmiHeader.biWidth = I_Width
    bi.bmiHeader.biHeight = I_Height
    bi.bmiHeader.biPlanes = 1           ' ColorPlanes
    bi.bmiHeader.biBitCount = BxP       ' BitsPerPixel * ColorPlanes
    bi.bmiHeader.biCompression = BI_RGB
    bi.bmiHeader.biSizeImage = 0
    bi.bmiHeader.biXPelsPerMeter = 0
    bi.bmiHeader.biYPelsPerMeter = 0
    bi.bmiHeader.biClrUsed = 0
    bi.bmiHeader.biClrImportant = 0
'
    ' Calculate the number of bytes needed to store the image:
    XBMP = ScanAlign(3 * I_Width)
    BytesNeeded = XBMP * I_Height
    ReDim MPix3(0 To XBMP - 1, 0 To I_Height - 1)
'
    ' L' immagine deve essere in background
    ' Non necessario se PBSrc.Autoredraw = True:
    'If Not BackTrasPB(PBSrc) Then Err.Raise 1001, , "BackTrasPB"
'
    ' Get the bits that make up the image:
    R1 = GetDIBits(hDC_Mem, PBSrc.Picture, 0, I_Height _
                 , MPix3(0, 0), bi, DIB_RGB_COLORS)
    If R1 = 0 Then
        M$ = "Errore in GetDIBits." & vbNewLine
        Err.Raise 1001, , "GetDIBits"
    End If
'
'
LeggeBitMap3PB_ERR:
    LeggeBitMap3PB = (Err = 0)
    If Err <> 0 Then
        M$ = M$ & "Errore N. " & Err.Number & vbNewLine & Err.Description
        MsgBox M$, vbCritical, " Errore in LeggeBitMap3PB"
    End If
'
    R1 = DeleteDC(hDC_Mem)
'
'
'
End Function
Public Function LeggeBitMap4PB(ByVal PBSrc As PictureBox, MPix4() As Long _
    , Optional ByRef I_Width&, Optional ByRef I_Height& _
    , Optional ByRef BxP&) As Boolean
'
'   Legge l' immagine del PictureBox PBSrc nella matrice MPix4().
'   I Long nella matrice sono i colori RGB dei pixels di PBSrc con il byte del
'   rosso scambiato con quello del blu.
'   Ritorna anche:
'    I_Width:   larghezza di una riga dell' immagine [Pixels].
'    I_Height:  altezza dell' immagine [Pixels].
'    BxP:       profondita' di colore dell' immagine [BitsPerPixel]
'               sempre = 32.
'
    Dim R1&, M$
    Dim hDC_PB&, hDC_Mem&
    Dim Rect As RECT_Type
    Dim bi As BITMAPINFO_Type
'
    On Error GoTo LeggeBitMap4PB_ERR
'
    ' Create a temporary memory DC and select into it
    ' the background picture of the PBSrc Picture Box:
    hDC_PB = GetWindowDC(PBSrc.hWnd)
    hDC_Mem = CreateCompatibleDC(hDC_PB)
'
    ' Get the size of the picture bitmap:
    R1 = GetClientRect(PBSrc.hWnd, Rect)
    I_Width = Abs(Rect.Right - Rect.Left)
    I_Height = Abs(Rect.Bottom - Rect.Top)
'
    ' Fill the BITMAPINFOHEADER for the desired DIB:
    BxP = 32
    bi.bmiHeader.biSize = Len(bi.bmiHeader)
    bi.bmiHeader.biWidth = I_Width
    bi.bmiHeader.biHeight = I_Height
    bi.bmiHeader.biPlanes = 1           ' ColorPlanes
    bi.bmiHeader.biBitCount = BxP       ' BitsPerPixel * ColorPlanes
    bi.bmiHeader.biCompression = BI_RGB
    bi.bmiHeader.biSizeImage = 0
    bi.bmiHeader.biXPelsPerMeter = 0
    bi.bmiHeader.biYPelsPerMeter = 0
    bi.bmiHeader.biClrUsed = 0
    bi.bmiHeader.biClrImportant = 0
'
    ' Calculate the number of bytes needed to store the image:
    ReDim MPix4(0 To I_Width - 1, 0 To I_Height - 1)
'
    ' L' immagine deve essere in background
    ' Non necessario se PBSrc.Autoredraw = True:
    'If Not BackTrasPB(PBSrc) Then Err.Raise 1001, , "BackTrasPB"
'
    ' Get the bits that make up the image:
    R1 = GetDIBits(hDC_Mem, PBSrc.Picture, 0, I_Height _
                 , MPix4(0, 0), bi, DIB_RGB_COLORS)
    If R1 = 0 Then
        ApiError
        M$ = "Errore in GetDIBits." & vbNewLine
        Err.Raise 1001, , "GetDIBits"
    End If
'
'
LeggeBitMap4PB_ERR:
    LeggeBitMap4PB = (Err = 0)
    If Err <> 0 Then
        M$ = M$ & "Errore N. " & Err.Number & vbNewLine & Err.Description
        MsgBox M$, vbCritical, " Errore in LeggeBitMap4PB"
    End If
'
    R1 = DeleteDC(hDC_Mem)
'
'
'
End Function
Public Function ScriveBitMap3PB(MPix3() As Byte, ByVal PBDst As PictureBox _
    , Optional ByRef I_Width&, Optional ByRef I_Height&) As Boolean
'
'   Scrive la matrice MPix3() sull' immagine del PictureBox PBDst.
'   I bytes nella matrice sono a 3 a 3 i colori dei pixels da
'   scrivere su PBDst.
'   E.g.    MPix3(0, 0) = componente blu   del primo pixel dell' ultima riga in basso.
'           MPix3(1, 0) = componente verde  "    "     "     "     "     "    "   "
'           MPix3(2, 0) = componente rossa  "    "     "     "     "     "    "   "
'   Ritorna anche:
'    I_Width:  larghezza, in [Pixels], dell' immagine risultante.
'    I_Height: altezza, in [Pixels], dell' immagine risultante.
'
    Dim R1&, M$
    Dim BxP&, bi As BITMAPINFO_Type
'
    On Error GoTo ScriveBitMap3PB_ERR
'
    I_Width = ((UBound(MPix3, 1) + 1) \ 3)
    I_Height = UBound(MPix3, 2) + 1
'
    ' Fill the BITMAPINFOHEADER for the desired DIB:
    BxP = 24
    bi.bmiHeader.biSize = Len(bi.bmiHeader)
    bi.bmiHeader.biWidth = I_Width
    bi.bmiHeader.biHeight = I_Height
    bi.bmiHeader.biPlanes = 1           ' ColorPlanes
    bi.bmiHeader.biBitCount = BxP       ' BitsPerPixel * ColorPlanes
    bi.bmiHeader.biCompression = BI_RGB
    bi.bmiHeader.biSizeImage = 0
    bi.bmiHeader.biXPelsPerMeter = 0
    bi.bmiHeader.biYPelsPerMeter = 0
    bi.bmiHeader.biClrUsed = 0
    bi.bmiHeader.biClrImportant = 0
'
    R1 = SetDIBitsToDevice(PBDst.hDC, 0, 0, I_Width, I_Height _
                         , 0, 0, 0, I_Height, MPix3(0, 0), bi _
                         , DIB_RGB_COLORS)
    If R1 = 0 Then
        M$ = "Errore in SetDIBitsToDevice." & vbNewLine
        Err.Raise 1001, , "SetDIBitsToDevice"
    End If
'
'
ScriveBitMap3PB_ERR:
    ScriveBitMap3PB = (Err = 0)
    If Err <> 0 Then
        M$ = M$ & "Errore N. " & Err.Number & vbNewLine & Err.Description
        MsgBox M$, vbCritical, " Errore in ScriveBitMap3PB"
    End If
'
'
'
End Function
Public Function ScriveBitMap4PB(MPix4() As Long, ByVal PBDst As PictureBox _
    , Optional ByRef I_Width&, Optional ByRef I_Height&) As Boolean
'
'   Scrive la matrice MPix4() sull' immagine del PictureBox PBDst.
'   I Long nella matrice sono i colori RGB dei pixels di PBSrc con il byte del
'   rosso scambiato con quello del blu.
'   Ritorna anche:
'    I_Width:  larghezza, in [Pixels], dell' immagine risultante.
'    I_Height: altezza, in [Pixels], dell' immagine risultante.
'
    Dim R1&, M$
    Dim BxP&, bi As BITMAPINFO_Type
'
    On Error GoTo ScriveBitMap4PB_ERR
'
    I_Width = UBound(MPix4, 1) + 1
    I_Height = UBound(MPix4, 2) + 1
'
    ' Fill the BITMAPINFOHEADER for the desired DIB:
    BxP = 32
    bi.bmiHeader.biSize = Len(bi.bmiHeader)
    bi.bmiHeader.biWidth = I_Width
    bi.bmiHeader.biHeight = I_Height
    bi.bmiHeader.biPlanes = 1           ' ColorPlanes
    bi.bmiHeader.biBitCount = BxP       ' BitsPerPixel * ColorPlanes
    bi.bmiHeader.biCompression = BI_RGB
    bi.bmiHeader.biSizeImage = 0
    bi.bmiHeader.biXPelsPerMeter = 0
    bi.bmiHeader.biYPelsPerMeter = 0
    bi.bmiHeader.biClrUsed = 0
    bi.bmiHeader.biClrImportant = 0
'
    R1 = SetDIBitsToDevice(PBDst.hDC, 0, 0, I_Width, I_Height _
                         , 0, 0, 0, I_Height, MPix4(0, 0), bi _
                         , DIB_RGB_COLORS)
    If R1 = 0 Then
        M$ = "Errore in SetDIBitsToDevice." & vbNewLine
        Err.Raise 1001, , "SetDIBitsToDevice"
    End If
'
'
ScriveBitMap4PB_ERR:
    ScriveBitMap4PB = (Err = 0)
    If Err <> 0 Then
        M$ = M$ & "Errore N. " & Err.Number & vbNewLine & Err.Description
        MsgBox M$, vbCritical, " Errore in ScriveBitMap4PB"
    End If
'
'
'
End Function

